==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/helper/ItemNBTHelper.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.helper;

import com.google.gson.JsonObject;
import com.mojang.serialization.Dynamic;
import com.mojang.serialization.JsonOps;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_2499;
import net.minecraft.class_2509;
import net.minecraft.class_2520;
import org.jetbrains.annotations.Nullable;

import vazkii.botania.api.mana.ManaItem;
import vazkii.botania.xplat.XplatAbstractions;

public final class ItemNBTHelper {

	private static final int[] EMPTY_INT_ARRAY = new int[0];

	// SETTERS ///////////////////////////////////////////////////////////////////

	public static void set(class_1799 stack, String tag, class_2520 nbt) {
		stack.method_7948().method_10566(tag, nbt);
	}

	public static void setBoolean(class_1799 stack, String tag, boolean b) {
		stack.method_7948().method_10556(tag, b);
	}

	public static void setByte(class_1799 stack, String tag, byte b) {
		stack.method_7948().method_10567(tag, b);
	}

	public static void setShort(class_1799 stack, String tag, short s) {
		stack.method_7948().method_10575(tag, s);
	}

	public static void setInt(class_1799 stack, String tag, int i) {
		stack.method_7948().method_10569(tag, i);
	}

	public static void setIntArray(class_1799 stack, String tag, int[] val) {
		stack.method_7948().method_10539(tag, val);
	}

	public static void setLong(class_1799 stack, String tag, long l) {
		stack.method_7948().method_10544(tag, l);
	}

	public static void setFloat(class_1799 stack, String tag, float f) {
		stack.method_7948().method_10548(tag, f);
	}

	public static void setDouble(class_1799 stack, String tag, double d) {
		stack.method_7948().method_10549(tag, d);
	}

	public static void setCompound(class_1799 stack, String tag, class_2487 cmp) {
		if (!tag.equalsIgnoreCase("ench")) // not override the enchantments
		{
			stack.method_7948().method_10566(tag, cmp);
		}
	}

	public static void setString(class_1799 stack, String tag, String s) {
		stack.method_7948().method_10582(tag, s);
	}

	public static void setList(class_1799 stack, String tag, class_2499 list) {
		stack.method_7948().method_10566(tag, list);
	}

	public static void removeEntry(class_1799 stack, String tag) {
		stack.method_7983(tag);
	}

	// GETTERS ///////////////////////////////////////////////////////////////////

	public static boolean verifyExistance(class_1799 stack, String tag) {
		return !stack.method_7960() && stack.method_7985() && stack.method_7948().method_10545(tag);
	}

	@Nullable
	public static class_2520 get(class_1799 stack, String tag) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10580(tag) : null;
	}

	public static boolean getBoolean(class_1799 stack, String tag, boolean defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10577(tag) : defaultExpected;
	}

	public static byte getByte(class_1799 stack, String tag, byte defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10571(tag) : defaultExpected;
	}

	public static short getShort(class_1799 stack, String tag, short defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10568(tag) : defaultExpected;
	}

	public static int getInt(class_1799 stack, String tag, int defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10550(tag) : defaultExpected;
	}

	public static int[] getIntArray(class_1799 stack, String tag) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10561(tag) : EMPTY_INT_ARRAY;
	}

	public static long getLong(class_1799 stack, String tag, long defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10537(tag) : defaultExpected;
	}

	public static float getFloat(class_1799 stack, String tag, float defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10583(tag) : defaultExpected;
	}

	public static double getDouble(class_1799 stack, String tag, double defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10574(tag) : defaultExpected;
	}

	/**
	 * If nullifyOnFail is true it'll return null if it doesn't find any
	 * compounds, otherwise it'll return a new one.
	 **/
	public static class_2487 getCompound(class_1799 stack, String tag, boolean nullifyOnFail) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10562(tag) : nullifyOnFail ? null : new class_2487();
	}

	public static String getString(class_1799 stack, String tag, String defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10558(tag) : defaultExpected;
	}

	public static class_2499 getList(class_1799 stack, String tag, int objtype, boolean nullifyOnFail) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10554(tag, objtype) : nullifyOnFail ? null : new class_2499();
	}

	// OTHER ///////////////////////////////////////////////////////////////////

	/**
	 * Returns the fullness of the mana item:
	 * 0 if empty, 1 if partially full, 2 if full.
	 */
	public static int getFullness(ManaItem item) {
		int mana = item.getMana();
		if (mana == 0) {
			return 0;
		} else if (mana == item.getMaxMana()) {
			return 2;
		} else {
			return 1;
		}
	}

	public static class_1799 duplicateAndClearMana(class_1799 stack) {
		class_1799 copy = stack.method_7972();
		ManaItem manaItem = XplatAbstractions.INSTANCE.findManaItem(copy);
		if (manaItem != null) {
			manaItem.addMana(-manaItem.getMana());
		}
		return copy;
	}

	/**
	 * Checks if two items are the same and have the same NBT. If they are `IManaItems`, their mana property is matched
	 * on whether they are empty, partially full, or full.
	 */
	public static boolean matchTagAndManaFullness(class_1799 stack1, class_1799 stack2) {
		if (!class_1799.method_7984(stack1, stack2)) {
			return false;
		}
		ManaItem manaItem1 = XplatAbstractions.INSTANCE.findManaItem(stack1);
		ManaItem manaItem2 = XplatAbstractions.INSTANCE.findManaItem(stack2);
		if (manaItem1 != null && manaItem2 != null) {
			if (getFullness(manaItem1) != getFullness(manaItem2)) {
				return false;
			} else {
				return class_1799.method_7975(duplicateAndClearMana(stack1), duplicateAndClearMana(stack2));
			}
		}
		return class_1799.method_7975(stack1, stack2);
	}

	/**
	 * Serializes the given stack such that {@link net.minecraft.class_1869#method_35228}
	 * would be able to read the result back
	 */
	public static JsonObject serializeStack(class_1799 stack) {
		class_2487 nbt = stack.method_7953(new class_2487());
		byte c = nbt.method_10571("Count");
		if (c != 1) {
			nbt.method_10567("count", c);
		}
		nbt.method_10551("Count");
		renameTag(nbt, "id", "item");
		renameTag(nbt, "tag", "nbt");
		Dynamic<class_2520> dyn = new Dynamic<>(class_2509.field_11560, nbt);
		return dyn.convert(JsonOps.INSTANCE).getValue().getAsJsonObject();
	}

	public static void renameTag(class_2487 nbt, String oldName, String newName) {
		class_2520 tag = nbt.method_10580(oldName);
		if (tag != null) {
			nbt.method_10551(oldName);
			nbt.method_10566(newName, tag);
		}
	}
}

==> Botania-1.20.1-451-FABRIC-20251019.115504-18-sources.jar: vazkii/botania/common/helper/ItemNBTHelper.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.helper;

import com.google.gson.JsonObject;
import com.mojang.serialization.Dynamic;
import com.mojang.serialization.JsonOps;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_2499;
import net.minecraft.class_2509;
import net.minecraft.class_2520;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.Nullable;

import vazkii.botania.api.mana.ManaItem;
import vazkii.botania.xplat.XplatAbstractions;

public final class ItemNBTHelper {

	private static final int[] EMPTY_INT_ARRAY = new int[0];
	private static final long[] EMPTY_LONG_ARRAY = new long[0];

	// SETTERS ///////////////////////////////////////////////////////////////////

	public static void set(class_1799 stack, String tag, class_2520 nbt) {
		stack.method_7948().method_10566(tag, nbt);
	}

	public static void setBoolean(class_1799 stack, String tag, boolean b) {
		stack.method_7948().method_10556(tag, b);
	}

	public static void setByte(class_1799 stack, String tag, byte b) {
		stack.method_7948().method_10567(tag, b);
	}

	public static void setShort(class_1799 stack, String tag, short s) {
		stack.method_7948().method_10575(tag, s);
	}

	public static void setInt(class_1799 stack, String tag, int i) {
		stack.method_7948().method_10569(tag, i);
	}

	public static void setIntArray(class_1799 stack, String tag, int[] val) {
		stack.method_7948().method_10539(tag, val);
	}

	public static void setLong(class_1799 stack, String tag, long l) {
		stack.method_7948().method_10544(tag, l);
	}

	public static void setLongArray(class_1799 stack, String tag, long[] val) {
		stack.method_7948().method_10564(tag, val);
	}

	public static void setFloat(class_1799 stack, String tag, float f) {
		stack.method_7948().method_10548(tag, f);
	}

	public static void setDouble(class_1799 stack, String tag, double d) {
		stack.method_7948().method_10549(tag, d);
	}

	public static void setCompound(class_1799 stack, String tag, class_2487 cmp) {
		if (!tag.equalsIgnoreCase("ench")) // not override the enchantments
		{
			stack.method_7948().method_10566(tag, cmp);
		}
	}

	public static void setString(class_1799 stack, String tag, String s) {
		stack.method_7948().method_10582(tag, s);
	}

	public static void setList(class_1799 stack, String tag, class_2499 list) {
		stack.method_7948().method_10566(tag, list);
	}

	public static void removeEntry(class_1799 stack, String tag) {
		stack.method_7983(tag);
	}

	// GETTERS ///////////////////////////////////////////////////////////////////

	public static boolean verifyExistance(class_1799 stack, String tag) {
		return !stack.method_7960() && stack.method_7985() && stack.method_7948().method_10545(tag);
	}

	public static boolean verifyType(class_1799 stack, String tag, Class<? extends class_2520> tagClass) {
		return !stack.method_7960() && stack.method_7985() && tagClass.isInstance(stack.method_7948().method_10580(tag));
	}

	@Nullable
	public static class_2520 get(class_1799 stack, String tag) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10580(tag) : null;
	}

	public static boolean getBoolean(class_1799 stack, String tag, boolean defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10577(tag) : defaultExpected;
	}

	public static byte getByte(class_1799 stack, String tag, byte defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10571(tag) : defaultExpected;
	}

	public static short getShort(class_1799 stack, String tag, short defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10568(tag) : defaultExpected;
	}

	public static int getInt(class_1799 stack, String tag, int defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10550(tag) : defaultExpected;
	}

	public static int[] getIntArray(class_1799 stack, String tag) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10561(tag) : EMPTY_INT_ARRAY;
	}

	public static long getLong(class_1799 stack, String tag, long defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10537(tag) : defaultExpected;
	}

	public static long[] getLongArray(class_1799 stack, String tag) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10565(tag) : EMPTY_LONG_ARRAY;
	}

	public static float getFloat(class_1799 stack, String tag, float defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10583(tag) : defaultExpected;
	}

	public static double getDouble(class_1799 stack, String tag, double defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10574(tag) : defaultExpected;
	}

	/**
	 * If nullifyOnFail is true it'll return null if it doesn't find any
	 * compounds, otherwise it'll return a new one.
	 **/
	@Nullable
	@Contract("_, _, false -> !null")
	public static class_2487 getCompound(class_1799 stack, String tag, boolean nullifyOnFail) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10562(tag) : nullifyOnFail ? null : new class_2487();
	}

	@Nullable
	@Contract("_, _, !null -> !null")
	public static String getString(class_1799 stack, String tag, String defaultExpected) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10558(tag) : defaultExpected;
	}

	@Nullable
	@Contract("_, _, _, false -> !null")
	public static class_2499 getList(class_1799 stack, String tag, int objtype, boolean nullifyOnFail) {
		return verifyExistance(stack, tag) ? stack.method_7948().method_10554(tag, objtype) : nullifyOnFail ? null : new class_2499();
	}

	// OTHER ///////////////////////////////////////////////////////////////////

	/**
	 * Returns the fullness of the mana item:
	 * 0 if empty, 1 if partially full, 2 if full.
	 */
	public static int getFullness(ManaItem item) {
		int mana = item.getMana();
		if (mana == 0) {
			return 0;
		} else if (mana == item.getMaxMana()) {
			return 2;
		} else {
			return 1;
		}
	}

	public static class_1799 duplicateAndClearMana(class_1799 stack) {
		class_1799 copy = stack.method_7972();
		ManaItem manaItem = XplatAbstractions.INSTANCE.findManaItem(copy);
		if (manaItem != null) {
			manaItem.addMana(-manaItem.getMana());
		}
		return copy;
	}

	/**
	 * Checks if two items are the same and have the same NBT. If they are `IManaItems`, their mana property is matched
	 * on whether they are empty, partially full, or full.
	 */
	public static boolean matchTagAndManaFullness(class_1799 stack1, class_1799 stack2) {
		if (!class_1799.method_7984(stack1, stack2)) {
			return false;
		}
		ManaItem manaItem1 = XplatAbstractions.INSTANCE.findManaItem(stack1);
		ManaItem manaItem2 = XplatAbstractions.INSTANCE.findManaItem(stack2);
		if (manaItem1 != null && manaItem2 != null) {
			if (getFullness(manaItem1) != getFullness(manaItem2)) {
				return false;
			} else {
				return class_1799.method_7973(duplicateAndClearMana(stack1), duplicateAndClearMana(stack2));
			}
		}
		return class_1799.method_31577(stack1, stack2);
	}

	/**
	 * Serializes the given stack such that {@link net.minecraft.class_1869#method_35228}
	 * would be able to read the result back
	 */
	public static JsonObject serializeStack(class_1799 stack) {
		class_2487 nbt = stack.method_7953(new class_2487());
		byte c = nbt.method_10571("Count");
		if (c != 1) {
			nbt.method_10567("count", c);
		}
		nbt.method_10551("Count");
		renameTag(nbt, "id", "item");
		renameTag(nbt, "tag", "nbt");
		Dynamic<class_2520> dyn = new Dynamic<>(class_2509.field_11560, nbt);
		return dyn.convert(JsonOps.INSTANCE).getValue().getAsJsonObject();
	}

	public static void renameTag(class_2487 nbt, String oldName, String newName) {
		class_2520 tag = nbt.method_10580(oldName);
		if (tag != null) {
			nbt.method_10551(oldName);
			nbt.method_10566(newName, tag);
		}
	}
}
